// Copyright 2011 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.api.ads.common.lib.utils;

import com.google.common.io.ByteStreams;
import com.google.common.io.InputSupplier;
import com.google.common.io.OutputSupplier;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

/**
 * Helper class to perform stream operations.
 *
 * @author Kevin Winter
 */
public final class Streams {

  /**
   * Default constructor.
   */
  private Streams() {}

  /**
   * Wraps the provided string into an InputStream.
   *
   * @param str String to wrap.
   * @return InputStream containing the string argument.
   */
  public static InputStream wrapString(String str) {
    try {
      return new ByteArrayInputStream(str.getBytes("UTF-8"));
    } catch (UnsupportedEncodingException e) {
      throw new IllegalStateException("UTF-8 should be supported.");
    }
  }

  /**
   * Reads all input into memory, close the steam, and return as a String.
   *
   * @param inputStream InputStream to read from.
   * @return String contents of the stream.
   * @throws IOException if there is an problem reading from the stream.
   */
  public static String readAll(InputStream inputStream) throws IOException {
    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
    copy(inputStream, byteArrayOutputStream);
    return new String(byteArrayOutputStream.toByteArray());
  }

  /**
   * Writes the specified string to stream with buffering and closes the stream.
   *
   * @param str String to write.
   * @param outputStream Stream to write to.
   * @throws IOException If there is an exception while writing.
   */
  public static void write(String str, final OutputStream outputStream) throws IOException {
    ByteStreams.write(str.getBytes(), new OutputSupplier<OutputStream>() {
      public OutputStream getOutput() {
        return outputStream;
      }
    });
  }

  /**
   * Copies the {@code inputStream} into the {@code outputSteam} and finally
   * closes the both streams.
   */
  public static void copy(final InputStream inputStream, final OutputStream outputStream)
      throws IOException {
    ByteStreams.copy(new InputSupplier<InputStream>() {
      public InputStream getInput() {
        return inputStream;
      }
    }, new OutputSupplier<OutputStream>() {
      public OutputStream getOutput() {
        return outputStream;
      }
    });
  }
}
